import time
from base64 import b64encode
from decimal import Decimal
from random import random
from nacl.bindings.crypto_sign import crypto_sign

from umipy import bech32
from umipy.enums import Prefix, SendVersion


class Transaction(list[int]):
    def set_version(self, version: int) -> None:
        self.append(version)

    def set_list(self, address: list[int]) -> None:
        self.extend(address)

    def set_amount(self, amount: Decimal) -> None:
        amount_to_bytes: bytes = int(amount).to_bytes(8, "big")
        to_list: list[int] = [int(i) for i in amount_to_bytes]
        self.extend(to_list)

    def sign(self, sk: list[int]) -> None:
        signature = crypto_sign(bytes(self), bytes(sk))
        self.extend(signature[:64])

    def sign_transaction(self, sk: list[int]) -> None:
        seconds = int(time.time()) - 10
        self.extend(to_4(seconds))
        self.extend(to_4(int(random() * 9999)))
        self.append(0)
        self.sign(sk=sk)


def to_public_key(address: str) -> list[int]:
    prefix, list_int, _ = bech32.bech32_decode(address)
    if list_int is None:
        raise ValueError(f"Invalid address, prefix is None {address=}")
    public_key = bech32.convertbits(list_int, 5, 8, False)
    return public_key


def prefix_to_version(prefix: Prefix) -> int | None:
    if prefix == "genesis":
        return 0

    if prefix == Prefix.UMI_TRADING:
        return 1

    if prefix == Prefix.UMI_STAKING:
        return 2

    if len(prefix) not in (3, 5):
        return None

    to_check = []
    for i in range(len(prefix)):
        modifier = 47 if any(char.isdigit() for char in prefix[i]) else 96
        to_check.append(ord(prefix[i]) - modifier)

    if len(to_check) > 3:
        return 32768 + ((to_check[3] << 5) + to_check[4])

    return (to_check[0] << 10) + (to_check[1] << 5) + to_check[2]


def to_2(value: int) -> list[int]:
    return [(value >> 8) & 0xFF, value & 0xFF]


def to_4(value: int) -> list[int]:
    return [
        (value & 0xFF000000) >> 24,
        (value & 0x00FF0000) >> 16,
        (value & 0x0000FF00) >> 8,
        (value & 0x000000FF) >> 0,
    ]


def transfer_addresses(
    private_key: list[int],
    from_address: str,
    to_address: str,
    amount: Decimal,
    send_version: int,
) -> str:
    trx = Transaction()
    trx.set_version(send_version)

    if len(to_address) == 62:
        slicer_to = 3
    elif len(to_address) == 67:  # umistake | umitrade
        slicer_to = 8
    else:
        slicer_to = 5

    token_to = to_address[:slicer_to]

    if len(from_address) == 62:
        slicer_from = 3
    elif len(to_address) == 67:  # umistake | umitrade
        slicer_from = 8
    else:
        slicer_from = 5

    token_from = from_address[:slicer_from]

    version_to = prefix_to_version(prefix=Prefix(token_to))
    version_from = prefix_to_version(prefix=Prefix(token_from))

    if version_to is None:
        raise ValueError(f"Invalid address, prefix is None {token_to=}")
    if version_from is None:
        raise ValueError(f"Invalid address, prefix is None {token_from=}")

    prefix_binary_to = to_2(version_to)
    prefix_binary_from = to_2(version_from)

    from_pk = to_public_key(from_address)
    from_addr = prefix_binary_from + from_pk
    trx.set_list(from_addr)

    to_pk = to_public_key(to_address)
    to_addr = prefix_binary_to + to_pk
    trx.set_list(to_addr)

    if send_version != SendVersion.LEGEND_MICRO:
        trx.set_amount(amount * Decimal("100"))
    else:
        trx.set_amount(amount * Decimal("1000000"))

    trx.sign_transaction(private_key)

    return b64encode(bytes(trx)).decode()



import random
import string


def get_random_string() -> str:
    length = random.randint(1, 200)
    letters = string.ascii_letters + string.digits
    return "".join(random.choice(letters) for _ in range(length))



import aiohttp
import pytest

from umipy import UmiPy


@pytest.fixture
async def umi_mainnet_v1():
    umi_ = UmiPy(session=aiohttp.ClientSession())
    yield umi_
    await umi_.close()


@pytest.fixture
async def umi_testnet_v1():
    umi_ = UmiPy(is_testnet=True, session=aiohttp.ClientSession())
    yield umi_
    await umi_.close()


@pytest.fixture
async def umi_legend_testnet():
    umi_ = UmiPy(is_testnet=True, is_legend=True, session=aiohttp.ClientSession())
    yield umi_
    await umi_.close()


@pytest.fixture
async def umi_legend_mainnet():
    umi_ = UmiPy(is_testnet=False, is_legend=True, session=aiohttp.ClientSession())
    yield umi_
    await umi_.close()



import asyncio

import aiohttp

from umipy import UmiPy


async def main():
    umi = UmiPy(session=aiohttp.ClientSession(), is_legend=True, is_testnet=False)

    res = await umi.get_transactions(
        address="umistake1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylswu0l6g",
        limit=10,
        offset=0,
    )
    print(res)
    await umi.close()


asyncio.run(main())



import pytest

from umipy import UmiPy


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "tx_hash, status, height, amount, message",
    [
        (
            "ba8ad045020342980676146ea03a70477f7923159005be037daf8e81616d746c",
            "error",
            None,
            None,
            "Not Found",
        ),
        (
            "6f5857caf39db233fd118b33b3e242a58332ac8644c458b5762a070e8bb49a2",
            "error",
            None,
            None,
            "encoding/hex: odd length hex string",
        ),
        (
            "f55049cdfaf33a1004c16b6cd381f81711993e6ca17ed7181f5c40ec4da10d51",
            "success",
            55765323,
            1_00,
            None,
        ),
    ],
)
async def test_get_tx_testnet(
    umi_testnet_v1: UmiPy,
    tx_hash: str,
    status: str,
    height: str | None,
    amount: str | None,
    message: str,
):
    transaction_data = await umi_testnet_v1.get_transaction(tx_hash)
    assert transaction_data.status == status

    if height:
        assert transaction_data.data is not None
        assert transaction_data.data.height == height

    if amount:
        assert transaction_data.data is not None
        assert transaction_data.data.amount == amount

    assert transaction_data.message == message


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "tx_hash, status, height, amount, message",
    [
        (
            "f55049cdfaf33a1004c16b6cd381f81711993e6ca17ed7181f5c40ec4da10d51",
            "success",
            55765323,
            100,
            None,
        ),
        (
            "6f3458c067c6ccc05bee8b5a30706c832a93ae1d965b93c2373ecf902f2403bd",
            "error",
            None,
            None,
            "Not Found",
        ),
        (
            "a7c96d1306889f7d211f1fb018239c9489f4396a19dd8a70bea83293c304a46d",
            "success",
            61653172,
            30_000_00,
            None,
        ),
        (
            "f55049cdfaf33a1004c16b6cd381f81711993e6ca17ed7181f5c40ec4da10d51",
            "success",
            55765323,
            1_00,
            None,
        ),
        (
            "6f3458c067c6ccc05bee8b5a30706c832a93ae1d965b93c2373ecf902f2403bd",
            "error",
            None,
            None,
            "Not Found",
        ),
        (
            "6f5857caf39db233fd118b33b3e242a58332ac8644c458b5762a070e8bb49a2",
            "error",
            None,
            None,
            "encoding/hex: odd length hex string",
        ),
    ],
)
async def test_get_tx(
    umi_mainnet_v1: UmiPy,
    tx_hash: str,
    status: str,
    height: str | None,
    amount: str | None,
    message: str,
):
    transaction_data = await umi_mainnet_v1.get_transaction(tx_hash)
    assert transaction_data.status == status

    if height:
        assert transaction_data.data is not None
        assert transaction_data.data.height == height

    if amount:
        assert transaction_data.data is not None
        assert transaction_data.data.amount == amount

    assert transaction_data.message == message


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "tx_hash, status, height, amount, message",
    [
        (
            "7bdd8b4f9cd79158bf77af2e6696248b611ae8545f6c5ac91d3e87dcd29ae6cb",
            "success",
            33,
            1,
            None,
        ),
        (
            "4a43760552510497ce135f6d0579e601f4ff8352ded5cf19f7e156a5dc73c2fe",
            "error",
            None,
            None,
            "Not Found",
        ),
        (
            "1fd090de0225510871a26b67458948d49b0e5de04295d0c1f8b3d607720c4924",
            "error",
            None,
            None,
            "Not Found",
        ),
        (
            "6fbb5a93fce2a6ba435c08157dd9c129234217fcf3711146b34da1963b6c1be",
            "error",
            None,
            None,
            "encoding/hex: odd length hex string",
        ),
    ],
)
async def test_get_tx_legend_testnet(
    umi_legend_testnet: UmiPy,
    tx_hash: str,
    status: str,
    height: str | None,
    amount: str | None,
    message: str,
):
    transaction_data = await umi_legend_testnet.get_transaction(tx_hash)
    print(tx_hash, transaction_data)

    assert transaction_data.status == status

    if height:
        assert transaction_data.data is not None
        assert transaction_data.data.height == height

    if amount:
        assert transaction_data.data is not None
        assert transaction_data.data.amount == amount

    assert transaction_data.message == message


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "tx_hash, status, height, amount, message",
    [
        (
            "7bdd8b4f9cd79158bf77af2e6696248b611ae8545f6c5ac91d3e87dcd29ae6cb",
            "success",
            33,
            1,
            None,
        )
    ],
)
async def test_get_tx_legend(
    umi_legend_mainnet: UmiPy,
    tx_hash: str,
    status: str,
    height: str | None,
    amount: str | None,
    message: str,
):
    transaction_data = await umi_legend_mainnet.get_transaction(tx_hash)

    assert transaction_data.status == status

    if height:
        assert transaction_data.data is not None
        assert transaction_data.data.height == height

    if amount:
        assert transaction_data.data is not None
        assert transaction_data.data.amount == amount

    assert transaction_data.message == message



import asyncio

import aiohttp

from umipy import UmiPy, Prefix


async def main():
    umi = UmiPy(session=aiohttp.ClientSession())
    my_mnemonic = "word1 word2..."
    old_wallet = umi.restore_wallet(my_mnemonic, prefix=Prefix.ROD_TRADING)
    # restore old wallet with balance by mnemonic

    new_wallet = umi.generate_wallet(prefix=Prefix.ROD_TRADING)
    print(new_wallet.mnemonic)
    balance = await umi.get_balance(new_wallet.address)
    print(balance)  # 0
    # create new wallet

    amount = 0.02
    print(f"send from {old_wallet.address} to {new_wallet.address} {amount} GLZ")
    send_result = await umi.send_addresses(
        private_key=old_wallet.keys.private_key,
        from_address=old_wallet.address,
        target_address=new_wallet.address,
        amount=amount,
    )
    print("send result -", send_result)
    await asyncio.sleep(1)
    balance = await umi.get_balance(new_wallet.address)
    print(balance)  # 0.01

    await umi.close()


asyncio.run(main())



import asyncio

import aiohttp

from umipy import UmiPy


async def main():
    umi = UmiPy(session=aiohttp.ClientSession())

    addresses = [
        "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
        "gls1kjmggptlgafkxaf3pqs473l893e928gnjlrhxntmlyvumdnvj8hsq07g4e",
        "gls1axur4xve407rsejjzr2ft75x80rh3s86w8s4sytflw6383fj4tjszvuh5y",
        "rod011xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylszx22gc",
        "rod1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls547jk6",
        "rod021xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls2mn9xn",
        "rod1fjl4nchvpt8xm30luacrt5s50g8uvn086fg5r4wfdc48s26uesfs34dvzm",
        "rod391fjl4nchvpt8xm30luacrt5s50g8uvn086fg5r4wfdc48s26uesfs2plhhz",
    ]

    addresses *= 100

    balances = await umi.get_balances_bulk(addresses=addresses)
    for balance in balances.items:
        print(balance.address, balance.balance)

    await umi.close()


asyncio.run(main())



import asyncio

import aiohttp

from umipy import UmiPy


async def main():
    umi = UmiPy(session=aiohttp.ClientSession())

    some_wallet = umi.generate_wallet()
    print(some_wallet.mnemonic)
    signed = umi.sign_message(some_wallet.keys.private_key, "123")
    print(signed)
    await umi.close()


asyncio.run(main())



from umipy import get_address_prefix


def main():
    address1 = "rod1t6q9zdg9wnmet3hc9fwulsqa4qhd52fg53eqdwnx2wccp2czmllq3t302y"
    print(get_address_prefix(address1))
    address2 = "umi1570tt8sq40k3cahxt0xleaw4l20rwwnzvvupmlu6pcue4mhgm9cslgwvqk"
    print(get_address_prefix(address2))
    address3 = "rod371xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls3c6kpn"
    print(get_address_prefix(address3))


main()



import pytest

from tests.utils import get_random_string
from umipy import UmiPy


@pytest.mark.asyncio
@pytest.mark.parametrize("message", [get_random_string() for _ in range(100)])
async def test_get_balance(umi_mainnet_v1: UmiPy, message: str):
    old_wallet = umi_mainnet_v1.generate_wallet()
    signed_message = umi_mainnet_v1.sign_message(
        message=message, private_key=old_wallet.keys.private_key
    )

    verify_result = umi_mainnet_v1.sign_verify(
        signature=signed_message, address=old_wallet.address, original_message=message
    )
    assert verify_result



import base64
from decimal import Decimal
from typing import Sequence, Any

import aiohttp
from nacl.bindings import crypto_sign, crypto_sign_open
from nacl.exceptions import BadSignatureError

from umipy.enums import BalanceType, Prefix
from umipy.exceptions import UmiApiException
from umipy.generate_wallet import generate_wallet, restore_wallet
from umipy.helpers import get_api_url, get_send_version
from umipy.models import (
    BalanceResponse,
    TransactionsResponse,
    WalletResponse,
    Keys,
    TransactionResponse,
    SendResponse,
    Transaction,
    BalancesResponse,
    BalanceAddressData,
)
from umipy.transfer import transfer_addresses, to_public_key


class UmiPy:
    def __init__(
        self,
        session: aiohttp.ClientSession,
        is_testnet: bool = False,
        is_legend: bool = False,
    ):
        base_url = get_api_url(is_testnet=is_testnet, is_legend=is_legend)
        self.base_url = base_url
        self.send_version = get_send_version(is_legend=is_legend)

        self.session = session

    async def request(
        self,
        method: str,
        path: str,
        params: dict[str, Any] | None = None,
        data: dict[str, Any] | str | None = None,
    ) -> Any:
        response = await self.session.request(
            method=method, url=f"{self.base_url}{path}", params=params, json=data
        )

        if response.status != 200:
            raise UmiApiException(f"Error {response.status} - {await response.text()}")

        json_response = await response.json()
        return json_response

    async def close(self) -> None:
        await self.session.close()

    async def get_balance(
        self, address: str, balance_type: BalanceType = BalanceType.confirmed
    ) -> BalanceResponse:
        response = await self.request("GET", f"/api/addresses/{address}/account")
        if "error" in response:
            return BalanceResponse(balance=0)

        if "confirmedBalanceMicro" in response["data"]:
            return BalanceResponse(
                balance=response["data"]["confirmedBalanceMicro"] / 1_000_000
            )

        return BalanceResponse(balance=response["data"][balance_type.value] / 100)

    async def get_balances_bulk(
        self,
        addresses: Sequence[str],
    ) -> BalancesResponse:
        response = await self.request(
            "POST", "/api/balances", data={"data": list(addresses)}
        )
        return BalancesResponse(
            items=list(
                map(
                    lambda balance: BalanceAddressData(
                        balance=balance["balance"] / 100, address=balance["address"]
                    ),
                    response["data"],
                )
            )
        )

    async def get_transactions(
        self, address: str, limit: int | None = None, offset: int | None = None
    ) -> TransactionsResponse:
        params = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        response = await self.request(
            "GET",
            f"/api/addresses/{address}/transactions",
            params=params,
        )

        if "error" in response:
            return TransactionsResponse(total_count=0, items=[])

        return TransactionsResponse(**response["data"])

    def generate_wallet(self, prefix: Prefix = Prefix.UMI) -> WalletResponse:
        address, mnemonic, public_key, private_key = generate_wallet(prefix=prefix)
        return WalletResponse(
            address=address,
            mnemonic=mnemonic,
            keys=Keys(
                public_key=public_key, private_key=private_key, key_type="банан беби"
            ),
        )

    async def send_addresses(
        self,
        private_key: list[int],
        from_address: str,
        target_address: str,
        amount: Decimal,
    ) -> SendResponse:
        encoded_data = transfer_addresses(
            private_key=private_key,
            from_address=from_address,
            to_address=target_address,
            amount=amount,
            send_version=self.send_version,
        )
        response = await self.request(
            method="POST", path="/api/mempool", data={"data": encoded_data}
        )

        return SendResponse(**response)

    def restore_wallet(
        self,
        mnemonic: str,
        prefix: Prefix = Prefix.UMI,
    ) -> WalletResponse:
        address, mnemonic, public_key, private_key = restore_wallet(
            mnemonic=mnemonic, prefix=prefix
        )
        return WalletResponse(
            address=address,
            mnemonic=mnemonic,
            keys=Keys(
                public_key=public_key, private_key=private_key, key_type="банан беби"
            ),
        )

    def sign_message(
        self,
        private_key: list[int],
        message: str,
    ) -> str:
        encoded_message = message.encode()
        result = crypto_sign(message=encoded_message, sk=bytes(private_key))

        return base64.b64encode(result[: -len(encoded_message)]).decode()

    def sign_verify(self, signature: str, original_message: str, address: str) -> bool:
        public_key = to_public_key(address)
        base64_sig = base64.b64decode(signature)

        base64_sig += original_message.encode()
        try:
            result = crypto_sign_open(signed=base64_sig, pk=bytes(public_key))
        except BadSignatureError:
            return False

        return result.decode() == original_message

    async def get_transaction(self, transaction_hash: str) -> TransactionResponse:
        response = await (
            await self.session.request(
                method="GET",
                url=f"{self.base_url}/api/transactions/{transaction_hash}",
            )
        ).json()
        if "error" in response:
            return TransactionResponse(
                status="error", code=500, message=response["error"]["message"]
            )
        return TransactionResponse(
            status="success", code=None, data=Transaction(**response["data"])
        )



from .api import UmiPy
from .enums import Prefix, BalanceType
from . import bech32
from .helpers import get_address_prefix
from .models import BalanceResponse, TransactionResponse

__all__ = (
    "UmiPy",
    "Prefix",
    "BalanceType",
    "bech32",
    "get_address_prefix",
    "BalanceResponse",
    "TransactionResponse",
)



import pytest

from umipy import Prefix, get_address_prefix


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "address, expected_prefix",
    [
        (
            "rod1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls547jk6",
            Prefix.ROD_TRADING,
        ),
        (
            "rod051xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls9rc78m",
            Prefix.KOSTENKO,
        ),
        (
            "rod131xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls5vae0x",
            Prefix.GUZMAN,
        ),
        (
            "rod191xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsdsejz4",
            Prefix.NEKRASOV,
        ),
        (
            "rod301xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls7q3dqm",
            Prefix.STAROSTENKO,
        ),
        (
            "rod361xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylswn2nud",
            Prefix.ROD_WALK,
        ),
        (
            "gls1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls3cmu2c",
            Prefix.GLIZE_STAKING,
        ),
        (
            "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
            Prefix.GLIZE,
        ),
        (
            "umi1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls83w7zj",
            Prefix.UMI,
        ),
        (
            "umi1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jyls83w7zj",
            Prefix.UMI,
        ),
        (
            "umistake1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylswu0l6g",
            Prefix.UMI_STAKING,
        ),
        (
            "umitrade1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsz49rj7",
            Prefix.UMI_TRADING,
        ),
        (
            "abctrade1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsz49rj7",
            None,
        ),
    ],
)
async def test_get_address_prefix(address: str, expected_prefix: Prefix):
    got_prefix = get_address_prefix(address)
    assert got_prefix == expected_prefix



import asyncio

import aiohttp

from umipy import UmiPy
from umipy.enums import BalanceType


async def main():
    umi = UmiPy(session=aiohttp.ClientSession())
    balance = await umi.get_balance(
        "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq"
    )
    print(balance)

    trans = await umi.get_transactions(
        "umi17ymaed9h9hq7s5pc2f5fhmlzpmsk3qtc6g2cgm360zysz0uvq44qnxlsuz"
    )
    print(trans)

    unconfirmed_balance = await umi.get_balance(
        "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
        balance_type=BalanceType.unconfirmed,
    )
    print(unconfirmed_balance)

    await umi.close()


asyncio.run(main())



import asyncio

import aiohttp

from umipy import UmiPy


async def main():
    umi = UmiPy(session=aiohttp.ClientSession(), is_testnet=True)

    bad_trans_data = await umi.get_transaction(
        transaction_hash="ba8ad045020342980676146ea03a70477f7923159005be037daf8e81616d746c"
    )
    print(f"v1 tesnet: bad_trans_data rod v1 many send - {bad_trans_data}\n")
    bad_trans_data = await umi.get_transaction(
        transaction_hash="6f5857caf39db233fd118b33b3e242a58332ac8644c458b5762a070e8bb49a2"
    )
    print(f"v1 tesnet: bad_trans_data rod v1 wrong hash - {bad_trans_data}\n")
    good_trans_data = await umi.get_transaction(
        transaction_hash="f55049cdfaf33a1004c16b6cd381f81711993e6ca17ed7181f5c40ec4da10d51"
    )
    print(f"v1 tesnet: good_trans_data rod v1 - {good_trans_data}\n")
    await umi.close()

    umi = UmiPy(session=aiohttp.ClientSession(), is_legend=True, is_testnet=True)

    bad_trans_data = await umi.get_transaction(
        transaction_hash="1fd090de0225510871a26b67458948d49b0e5de04295d0c1f8b3d607720c4924"
    )
    print(f"legend testnet: bad_trans_data umi legend many send - {bad_trans_data}\n")
    bad_trans_data2 = await umi.get_transaction(
        transaction_hash="6fbb5a93fce2a6ba435c08157dd9c129234217fcf3711146b34da1963b6c1be"
    )
    print(f"legend testnet: bad_trans_data umi legend wrong hash - {bad_trans_data2}\n")
    good_trans_data = await umi.get_transaction(
        transaction_hash="b870b0b52e8398fa50eb54f1a1d7f27ab10a425df13301c155641e3fe4d67d65"
    )
    print(f"legend testnet: good_trans_data umi legend - {good_trans_data}\n")

    await umi.close()

    umi = UmiPy(session=aiohttp.ClientSession(), is_legend=True, is_testnet=False)

    good_trans_data = await umi.get_transaction(
        transaction_hash="7bdd8b4f9cd79158bf77af2e6696248b611ae8545f6c5ac91d3e87dcd29ae6cb"
    )
    print(f"legend mainnet: good orig legend - {good_trans_data}\n")

    await umi.close()

    umi = UmiPy(session=aiohttp.ClientSession(), is_testnet=False)
    bad_trans_data = await umi.get_transaction(
        transaction_hash="48bd9721e245866b95629ed39a3768f7c8aa466e14dca858fa7a7555c3408bec"
    )
    print(f"v1 mainnet: good v1 - {bad_trans_data}\n")

    await umi.close()


asyncio.run(main())



import pytest

from umipy import UmiPy, Prefix


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "prefix",
    [
        Prefix.UMI,
        Prefix.GLIZE,
        Prefix.GLIZE_STAKING,
        Prefix.DOSTATOK,
        Prefix.ROD_TRADING,
    ],
)
async def test_get_balance(umi_mainnet_v1: UmiPy, prefix: Prefix):
    wallet = umi_mainnet_v1.generate_wallet(prefix=prefix)
    restored = umi_mainnet_v1.restore_wallet(mnemonic=wallet.mnemonic, prefix=prefix)

    assert wallet.mnemonic == restored.mnemonic
    assert wallet.address == restored.address
    assert wallet.keys == restored.keys



from unittest.mock import AsyncMock

import pytest

from umipy import UmiPy, BalanceType
from umipy.models import BalancesResponse


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "address, mock_response, balance_type, expected_balance",
    [
        (
            "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
            {"data": {"confirmedBalance": 754422}},
            BalanceType.confirmed,
            7544.22,
        ),
        (
            "umi1570tt8sq40k3cahxt0xleaw4l20rwwnzvvupmlu6pcue4mhgm9cslgwvqk",
            {"data": {"confirmedBalance": 0}},
            BalanceType.confirmed,
            0,
        ),
        (
            "rod201jy0wk999yfww58r5ykms4lsllg9p5jayzz0gtwxh7vx3e948s4qsgassg9",
            {"data": {"confirmedBalance": 100}},
            BalanceType.confirmed,
            1,
        ),
        (
            "rod011ug698khvxfykmqjhyn0culjzwmu2a0q208shatrc35hugf0uv4zq95chg4",
            {"data": {"confirmedBalance": 12345}},
            BalanceType.confirmed,
            123.45,
        ),
        (
            "rod011ug698khvxfykmqjhyn0culjzwmu2a0q208shatrc35hugf0uv4zq95chg4",
            {"data": {"unconfirmedBalance": 12345}},
            BalanceType.unconfirmed,
            123.45,
        ),
        (
            "rod011ug698khvxfykmqjhyn0culjzwmu2a0q208shatrc35hugf0uv4zq95chg4",
            {"data": {"unconfirmedBalance": 0}},
            BalanceType.unconfirmed,
            0,
        ),
        (
            "rod011ug698khvxfykmqjhyn0culjzwmu2a0q208shatrc35hugf0uv4zq95chg4",
            {"data": {"unconfirmedBalance": 1}},
            BalanceType.unconfirmed,
            0.01,
        ),
        (
            "rod011ug698khvxfykmqjhyn0culjzwmu2a0q208shatrc35hugf0uv4zq95chg4",
            {"data": {"unconfirmedBalance": 100000}},
            BalanceType.unconfirmed,
            1000,
        ),
    ],
)
async def test_get_confirmed_balance(
    mocker,
    umi_mainnet_v1: UmiPy,
    address: str,
    mock_response: dict,
    balance_type: BalanceType,
    expected_balance: float,
):
    mocker.patch.object(
        umi_mainnet_v1, "request", AsyncMock(return_value=mock_response)
    )

    if balance_type is BalanceType.confirmed:
        assert (
            await umi_mainnet_v1.get_balance(address=address)
        ).balance == expected_balance

    assert (
        await umi_mainnet_v1.get_balance(address=address, balance_type=balance_type)
    ).balance == expected_balance


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "addresses, mock_response, response_len",
    [
        (
            [
                "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
                "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
            ],
            {
                "items": [
                    {
                        "address": "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
                        "balance": 754422,
                    },
                    {
                        "address": "glz1xhgwhyczqwphzh3zzluqch9j4uxargp6pnnn0fd9wgy7hq79jylsqexcfq",
                        "balance": 754422,
                    },
                ]
            },
            2,
        )
    ],
)
async def test_get_bulk_balance(
    mocker,
    umi_mainnet_v1: UmiPy,
    addresses: list[str],
    mock_response: dict,
    response_len: float,
):
    mocker.patch.object(
        umi_mainnet_v1, "request", AsyncMock(return_value=mock_response)
    )

    response = BalancesResponse(**mock_response)

    assert len(response.items) == response_len



import asyncio

import aiohttp

from umipy import UmiPy, Prefix


async def main():
    umi = UmiPy(session=aiohttp.ClientSession())

    my_mnemonic = "word1 word2 ..."
    old_wallet = umi.restore_wallet(my_mnemonic, prefix=Prefix.GLIZE)

    message = "test1234"
    signed_message = umi.sign_message(
        message=message, private_key=old_wallet.keys.private_key
    )
    print(signed_message)

    verify_result = umi.sign_verify(
        signature=signed_message, address=old_wallet.address, original_message=message
    )
    print(verify_result)

    await umi.close()


asyncio.run(main())
